@@ -26,10 +26,11 @@ from process import ProcessLifecycleHandler, SshProcess, ProcessOutputHandler, \
 from report import Report
 from ssh_utils import get_rewritten_host_address, get_scp_command, \
     get_taktuk_connector_command, get_ssh_command
-from utils import name_from_cmdline, find_exe, intr_cond_wait, intr_event_wait, format_exc
+from utils import name_from_cmdline, intr_cond_wait, intr_event_wait, get_port
+from traceback import format_exc
 from substitutions import get_caller_context, remote_substitute
 from time_utils import get_seconds, format_date
-import threading, time, pipes, tempfile, os, shutil
+import threading, time, pipes, tempfile, os, shutil, stat
 
 class ActionLifecycleHandler(object):
 
@@ -677,69 +678,70 @@ class TaktukRemote(Action):
         self._taktuk_hosts_order = []
         self._taktuk_commands = ()
         self._taktuk = None
-        # we can provide per-host user with taktuk, but we cannot
-        # provide per-host port or keyfile, so check that all hosts
-        # and connection_params have the same port / keyfile (or None)
-        actual_connection_params = make_connection_params(self.connection_params)
-        check_default_port = actual_connection_params['port']
-        check_default_keyfile = actual_connection_params['keyfile']
-        check_keyfiles = set()
-        check_ports = set()
-        hosts_with_explicit_user = set()
-        for host in self.hosts:
-            if host.user != None:
-                hosts_with_explicit_user.add(host)
-            if host.keyfile != None:
-                check_keyfiles.add(host.keyfile)
-            else:
-                check_keyfiles.add(check_default_keyfile)
-            if host.port != None:
-                check_ports.add(host.port)
-            else:
-                check_ports.add(check_default_port)
-        if len(check_keyfiles) > 1 or len(check_ports) > 1:
-            raise ValueError, "unable to provide more than one keyfile / port for taktuk remote connection"
-        global_keyfile = None
-        global_port = None
-        if len(check_keyfiles) == 1:
-            global_keyfile = list(check_keyfiles)[0]
-        if len(check_ports) == 1:
-            global_port = list(check_ports)[0]
-        self._gen_taktukprocesses()
-        self._gen_taktuk_commands(hosts_with_explicit_user)
-        self._taktuk_commands += ("quit",)
-        #handler = _TaktukRemoteOutputHandler(self)
-        taktuk_options_filehandle, taktuk_options_filename = tempfile.mkstemp(prefix = 'tmp_execo_taktuk_')
-        self._taktuk_commands = " ".join(self._taktuk_commands)
-        os.write(taktuk_options_filehandle, self._taktuk_commands + "\n")
-        os.close(taktuk_options_filehandle)
-        logger.debug("generated taktuk tmp cmd file %s with content:\n%s", taktuk_options_filename, self._taktuk_commands)
-        real_taktuk_cmdline = (actual_connection_params['taktuk'],)
-        real_taktuk_cmdline += actual_connection_params['taktuk_options']
-        real_taktuk_cmdline += ("-o", 'output="A $position # $line\\n"',
-                                "-o", 'error="B $position # $line\\n"',
-                                "-o", 'status="C $position # $line\\n"',
-                                "-o", 'connector="D $position # $peer_position # $line\\n"',
-                                "-o", 'state="E $position # $peer_position # $line # ".event_msg($line)."\\n"',
-                                "-o", 'info="F $position # $line\\n"',
-                                "-o", 'taktuk="G $position # $line\\n"',
-                                "-o", 'message="H $position # $line\\n"',
-                                "-o", 'default="I $position # $type > $line\\n"')
-        real_taktuk_cmdline += ("-c", " ".join(
-            get_taktuk_connector_command(keyfile = global_keyfile,
-                                         port = global_port,
-                                         connection_params = self.connection_params)))
-        real_taktuk_cmdline += ("-F", taktuk_options_filename)
-        real_taktuk_cmdline = " ".join([pipes.quote(arg) for arg in real_taktuk_cmdline])
-        real_taktuk_cmdline += " && rm -f " + taktuk_options_filename
-        self._taktuk = Process(real_taktuk_cmdline)
-        #self._taktuk.close_stdin = False
-        self._taktuk.shell = True
-        #self._taktuk.default_stdout_handler = False
-        #self._taktuk.default_stderr_handler = False
-        self._taktuk.stdout_handlers.append(self._taktuk_stdout_output_handler)
-        #self._taktuk.stderr_handlers.append(self._taktuk_stderr_output_handler)
-        self._taktuk.lifecycle_handlers.append(_TaktukLH(self))
+        if len(self.hosts) > 0:
+            # we can provide per-host user with taktuk, but we cannot
+            # provide per-host port or keyfile, so check that all hosts
+            # and connection_params have the same port / keyfile (or None)
+            actual_connection_params = make_connection_params(self.connection_params)
+            check_default_port = actual_connection_params['port']
+            check_default_keyfile = actual_connection_params['keyfile']
+            check_keyfiles = set()
+            check_ports = set()
+            hosts_with_explicit_user = set()
+            for host in self.hosts:
+                if host.user != None:
+                    hosts_with_explicit_user.add(host)
+                if host.keyfile != None:
+                    check_keyfiles.add(host.keyfile)
+                else:
+                    check_keyfiles.add(check_default_keyfile)
+                if host.port != None:
+                    check_ports.add(host.port)
+                else:
+                    check_ports.add(check_default_port)
+            if len(check_keyfiles) > 1 or len(check_ports) > 1:
+                raise ValueError, "unable to provide more than one keyfile / port for taktuk remote connection"
+            global_keyfile = None
+            global_port = None
+            if len(check_keyfiles) == 1:
+                global_keyfile = list(check_keyfiles)[0]
+            if len(check_ports) == 1:
+                global_port = list(check_ports)[0]
+            self._gen_taktukprocesses()
+            self._gen_taktuk_commands(hosts_with_explicit_user)
+            self._taktuk_commands += ("quit",)
+            #handler = _TaktukRemoteOutputHandler(self)
+            taktuk_options_filehandle, taktuk_options_filename = tempfile.mkstemp(prefix = 'tmp_execo_taktuk_')
+            self._taktuk_commands = " ".join(self._taktuk_commands)
+            os.write(taktuk_options_filehandle, self._taktuk_commands + "\n")
+            os.close(taktuk_options_filehandle)
+            logger.debug("generated taktuk tmp cmd file %s with content:\n%s", taktuk_options_filename, self._taktuk_commands)
+            real_taktuk_cmdline = (actual_connection_params['taktuk'],)
+            real_taktuk_cmdline += actual_connection_params['taktuk_options']
+            real_taktuk_cmdline += ("-o", 'output="A $position # $line\\n"',
+                                    "-o", 'error="B $position # $line\\n"',
+                                    "-o", 'status="C $position # $line\\n"',
+                                    "-o", 'connector="D $position # $peer_position # $line\\n"',
+                                    "-o", 'state="E $position # $peer_position # $line # ".event_msg($line)."\\n"',
+                                    "-o", 'info="F $position # $line\\n"',
+                                    "-o", 'taktuk="G $position # $line\\n"',
+                                    "-o", 'message="H $position # $line\\n"',
+                                    "-o", 'default="I $position # $type > $line\\n"')
+            real_taktuk_cmdline += ("-c", " ".join(
+                get_taktuk_connector_command(keyfile = global_keyfile,
+                                             port = global_port,
+                                             connection_params = self.connection_params)))
+            real_taktuk_cmdline += ("-F", taktuk_options_filename)
+            real_taktuk_cmdline = " ".join([pipes.quote(arg) for arg in real_taktuk_cmdline])
+            real_taktuk_cmdline += " && rm -f " + taktuk_options_filename
+            self._taktuk = Process(real_taktuk_cmdline)
+            #self._taktuk.close_stdin = False
+            self._taktuk.shell = True
+            #self._taktuk.default_stdout_handler = False
+            #self._taktuk.default_stderr_handler = False
+            self._taktuk.stdout_handlers.append(self._taktuk_stdout_output_handler)
+            #self._taktuk.stderr_handlers.append(self._taktuk_stderr_output_handler)
+            self._taktuk.lifecycle_handlers.append(_TaktukLH(self))
 
     def start(self):
         retval = super(TaktukRemote, self).start()
@@ -752,13 +754,15 @@ class TaktukRemote(Action):
 
     def kill(self):
         retval = super(TaktukRemote, self).kill()
-        self._taktuk.ignore_exit_code = self._taktuk.nolog_exit_code = True
-        self._taktuk.kill()
+        if self._taktuk:
+            self._taktuk.ignore_exit_code = self._taktuk.nolog_exit_code = True
+            self._taktuk.kill()
         return retval
 
     def wait(self, timeout = None):
         retval = super(TaktukRemote, self).wait(timeout)
-        self._taktuk.wait()
+        if self._taktuk:
+            self._taktuk.wait()
         return retval
 
 class Put(Remote):
@@ -1462,7 +1466,9 @@ class _ChainPutCopy(ParallelActions):
         else:
             return super(_ChainPutCopy, self).start()
 
-_execo_chainput = find_exe("execo-chainput")
+_execo_chainput = os.path.abspath(os.path.join(os.path.dirname(__file__), "execo-chainput"))
+if not os.path.isfile(_execo_chainput): _execo_chainput = None
+
 class ChainPut(SequentialActions):
 
     """Broadcast local files to several remote host, with an unencrypted, unauthenticated chain of host to host copies (idea taken from `kastafior <https://gforge.inria.fr/plugins/scmgit/cgi-bin/gitweb.cgi?p=kadeploy3/kadeploy3.git;a=tree;f=addons/kastafior;h=e5472ce9e800c80d9f54d1097ebbcba77f8ccd7a;hb=3.1.7>`_).
@@ -1535,6 +1541,7 @@ class ChainPut(SequentialActions):
             if not _execo_chainput:
                 raise EnvironmentError, "unable to find execo-chainput"
             shutil.copy2(_execo_chainput, chainscript_filename)
+            os.chmod(chainscript_filename, os.stat(chainscript_filename).st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH )
 
             preparechain = TaktukPut(self.hosts,
                                      [ chainhosts_filename, chainscript_filename ],
@@ -1544,6 +1551,7 @@ class ChainPut(SequentialActions):
 
             #previous_action = preparechain
             chains = []
+            port = get_port()
             for findex, f in enumerate(self.local_files):
 
                 autoremoveopt = ""
@@ -1557,7 +1565,7 @@ class ChainPut(SequentialActions):
                         actual_connection_params['nc'],
                         actual_connection_params['chainput_nc_client_timeout'],
                         actual_connection_params['chainput_nc_server_timeout'],
-                        actual_connection_params['chainput_port'],
+                        port,
                         actual_connection_params['chainput_host_retry'],
                         chain_retries,
                         actual_connection_params['chainput_try_delay'],
@@ -1577,7 +1585,7 @@ class ChainPut(SequentialActions):
                         actual_connection_params['nc'],
                         actual_connection_params['chainput_nc_client_timeout'],
                         actual_connection_params['chainput_nc_server_timeout'],
-                        actual_connection_params['chainput_port'],
+                        port,
                         actual_connection_params['chainput_host_retry'],
                         chain_retries,
                         actual_connection_params['chainput_try_delay'],
