@@ -24,7 +24,8 @@ from log import style, logger
 from pty import openpty
 from ssh_utils import get_ssh_command, get_rewritten_host_address
 from time_utils import format_unixts, get_seconds
-from utils import compact_output, name_from_cmdline, intr_cond_wait, format_exc
+from utils import compact_output, name_from_cmdline, intr_cond_wait, get_port
+from traceback import format_exc
 from report import Report
 import errno, os, re, shlex, signal, subprocess
 import threading, time, pipes
@@ -682,14 +683,15 @@ class Process(ProcessBase):
                                                 stderr = subprocess.PIPE,
                                                 close_fds = True,
                                                 shell = self.shell,
-                                                preexec_fn = os.setsid)
+                                                preexec_fn = lambda: os.setpgid(0, the_conductor.pgrp))
             else:
                 self.process = subprocess.Popen(self._actual_cmd(),
                                                 stdin = subprocess.PIPE,
                                                 stdout = subprocess.PIPE,
                                                 stderr = subprocess.PIPE,
                                                 close_fds = True,
-                                                shell = self.shell)
+                                                shell = self.shell,
+                                                preexec_fn = lambda: os.setpgid(0, the_conductor.pgrp))
             self.pid = self.process.pid
             if self._actual_close_stdin():
                 self.process.stdin.close()
@@ -1007,3 +1009,125 @@ def get_process(*args, **kwargs):
         del kwargs["host"]
         if "connection_params" in kwargs: del kwargs["connection_params"]
         return Process(*args, **kwargs)
+
+class port_forwarder_stderr_handler(ProcessOutputHandler):
+    """parses verbose ssh output to notify when forward port is actually open"""
+
+    def __init__(self, local_port, bind_address):
+        super(port_forwarder_stderr_handler, self).__init__()
+        regex = "^debug1: Local forwarding listening on %s port %i\.\s*$" % (
+            bind_address,
+            local_port)
+        self.pf_open_re = re.compile(regex)
+
+    def read_line(self, process, string, eof, error):
+        if self.pf_open_re.match(string):
+            process.forwarding.set()
+
+def get_port_forwarder(host,
+                       connection_params,
+                       remote_host,
+                       remote_port,
+                       local_port = None,
+                       bind_address = None):
+    """Create an ssh port forwarder process (ssh -L).
+
+    This port forwarding process opens a listening socket on
+    localhost, and forwards it to the given remote host / port on the
+    remote side of the ssh connection.
+
+    :param host: remote side of the ssh connection
+
+    :param connection_params: connection params for connecting to host
+
+    :param remote_host: the remote host to connect to on the remote
+      side
+
+    :param remote_port: the remote port to connect to on the remote
+      side
+
+    :param local_port: the port to use locally for the listening
+      socket. If None (the default), a port is automatically selected
+      with ``execo.utils.get_port``
+
+    :param bind_address: the bind address to use locally for the
+      listening socket. If None (the default), it uses 127.0.0.1, so
+      the socket is only available to localhost.
+
+    :returns: a tuple (SshProcess, local_port). The returned
+      SshProcess executes a long sleep, and needs to be started /
+      killed as usual. It is configured to show no log / error when
+      killed. The returned SshProcess has an additionnal instance
+      variable `forwarding` which is a `threading.Event` which can be
+      waited upon to be notified when the forwarding port is actually
+      open (beware: this member is not correctly reset if the
+      forwarder SshProcess is reset)
+    """
+    if not bind_address:
+        bind_address = "127.0.0.1"
+    if not local_port:
+        local_port = get_port()
+    pf_conn_parms = make_connection_params(connection_params)
+    pf_conn_parms['ssh_options'] = pf_conn_parms['ssh_options'] + (
+        '-v',
+        '-L',
+        '%s:%i:%s:%i' % (
+            bind_address,
+            local_port,
+            Host(remote_host).address,
+            remote_port
+            ))
+    pf = SshProcess("sleep 31536000",
+                    host,
+                    connection_params = pf_conn_parms)
+    pf.ignore_exit_code = pf.nolog_exit_code = True
+    pf.forwarding = threading.Event()
+    pf.stderr_handlers.append(port_forwarder_stderr_handler(local_port,
+                                                            bind_address))
+    return pf, local_port
+
+class PortForwarder():
+    """Context manager for port forwarders"""
+
+    def __init__(self,
+                 host,
+                 connection_params,
+                 remote_host,
+                 remote_port,
+                 local_port = None,
+                 bind_address = None):
+        """
+        for params: see documentation of
+        ``execo.process.get_port_forwarder``
+
+        When entering the context, it returns a tuple host, port to
+        connect to. The forwarded port is guaranteed to be
+        operational.
+
+        When leaving the context, it kills the port forwarder
+        background process.
+        """
+        self.__host = host
+        self.__connection_params = connection_params
+        self.__remote_host = remote_host
+        self.__remote_port = remote_port
+        self.__local_port = local_port
+        self.__bind_address = bind_address
+        self.__port_forwarder = None
+        self.__local_port = None
+
+    def __enter__(self):
+        self.__port_forwarder, self.__local_port = get_port_forwarder(
+            self.__host,
+            self.__connection_params,
+            self.__remote_host,
+            self.__remote_port,
+            self.__local_port,
+            self.__bind_address)
+        self.__port_forwarder.start()
+        self.__port_forwarder.forwarding.wait()
+        return "127.0.0.1", self.__local_port
+
+    def __exit__(self, t, v, traceback):
+        self.__port_forwarder.kill()
+        return False
