@@ -99,8 +99,8 @@ _ksoh_deployed_nodes_header_re1 = re.compile("^Nodes correctly deployed on clust
 _ksoh_undeployed_nodes_header_re1 = re.compile("^Nodes not correctly deployed on cluster \w+\s*$") # for kadeploy3 < 3.2
 _ksoh_deployed_nodes_header_re2 = re.compile("^The \w+ is successful on nodes\s*$")                # for kadeploy3 >= 3.2
 _ksoh_undeployed_nodes_header_re2 = re.compile("^The \w+ failed on nodes\s*$")                     # for kadeploy3 >= 3.2
-_ksoh_deployed_node_re = re.compile("^(\S+)\s*$")
-_ksoh_undeployed_node_re = re.compile("^(\S+)(\s+\(.*\))?\s*$")
+_ksoh_deployed_node_re = re.compile("^(\w+-\d+\.\w+\.grid5000\.fr)\s*$")
+_ksoh_undeployed_node_re = re.compile("^(\w+-\d+\.\w+\.grid5000\.fr)(\s+\(.*)?\s*$")
 
 class _KadeployStdoutHandler(ProcessOutputHandler):
 
@@ -137,16 +137,12 @@ class _KadeployStdoutHandler(ProcessOutputHandler):
                 host_address = so.group(1)
                 self.kadeployer.deployed_hosts.add(host_address)
                 self.kadeployer._frontend_processes[process]["deployed_hosts"].add(host_address)
-            else:
-                self._current_section = self._SECTION_NONE
         elif self._current_section == self._SECTION_UNDEPLOYED_NODES:
             so = _ksoh_undeployed_node_re.search(string)
             if so != None:
                 host_address = so.group(1)
                 self.kadeployer.undeployed_hosts.add(host_address)
                 self.kadeployer._frontend_processes[process]["undeployed_hosts"].add(host_address)
-            else:
-                self._current_section = self._SECTION_NONE
 
 class _KadeployStderrHandler(ProcessOutputHandler):
 
@@ -483,13 +479,13 @@ def deploy(deployment,
                              len(deployed_hosts),
                              len(undeployed_hosts)))
 
-    logger.debug(style.emph("deploy finished") + " in %i tries, %s", num_tries_done, format_seconds(time.time() - start_time))
-    logger.debug("deploy  duration  attempted  deployed     deployed     total     total")
-    logger.debug("                  deploys    as reported  as reported  already   still")
-    logger.debug("                             by kadeploy  by check     deployed  undeployed")
-    logger.debug("---------------------------------------------------------------------------")
+    logger.detail(style.emph("deploy finished") + " in %i tries, %s", num_tries_done, format_seconds(time.time() - start_time))
+    logger.detail("deploy  duration  attempted  deployed     deployed     total     total")
+    logger.detail("                  deploys    as reported  as reported  already   still")
+    logger.detail("                             by kadeploy  by check     deployed  undeployed")
+    logger.detail("---------------------------------------------------------------------------")
     for (deploy_index, deploy_stat) in enumerate(deploy_stats):
-        logger.debug("#%-5.5s  %-8.8s  %-9.9s  %-11.11s  %-11.11s  %-8.8s  %-10.10s",
+        logger.detail("#%-5.5s  %-8.8s  %-9.9s  %-11.11s  %-11.11s  %-8.8s  %-10.10s",
             deploy_index,
             format_seconds(deploy_stat[0]),
             deploy_stat[1],
