@@ -31,6 +31,10 @@ from execo.time_utils import timedelta_to_seconds, get_seconds, \
 from execo_g5k.api_utils import get_g5k_sites, get_g5k_clusters, get_cluster_site, \
     get_site_clusters, get_resource_attributes, get_host_cluster, get_host_site
 from threading import Thread, currentThread
+from charter import g5k_charter_time, get_next_charter_period
+from execo_g5k.utils import G5kAutoPortForwarder
+from execo_g5k.config import g5k_configuration
+from traceback import format_exc
 
 try:
     import matplotlib.pyplot as PLT
@@ -38,19 +42,12 @@ try:
 except ImportError:
     pass
 
-_retrieve_method = None
-
-if 'grid5000.fr' in getfqdn():
-    try:
-        import MySQLdb
-        _retrieve_method = 'MySQL'
-    except:
-        pass
-        _retrieve_method = 'API'
-else:
+try:
+    import MySQLdb
+    _retrieve_method = 'MySQL'
+except:
     _retrieve_method = 'API'
 
-
 def get_planning(elements = ['grid5000'], vlan = False, subnet = False, storage = False,
             out_of_chart = False, starttime = None, endtime = None):
     """Retrieve the planning of the elements (site, cluster) and others resources.
@@ -168,8 +165,7 @@ def compute_slots(planning, walltime, excluded_elements = None):
                     for host, host_planning in cluster_planning.iteritems():
                         host_free = False
                         for free_slot in host_planning['free']:
-                            if free_slot[0] <= limit and free_slot[1] >= limit \
-                                + get_seconds(walltime):
+                            if free_slot[0] <= limit and free_slot[1] >= limit + walltime:
                                 host_free = True
                         if host_free:
                             free_elements['grid5000'] += 1
@@ -183,8 +179,7 @@ def compute_slots(planning, walltime, excluded_elements = None):
                     if int(vlan.split('-')[1]) < 10:
                         kavlan_free = False
                         for free_slot in vlan_planning['free']:
-                            if free_slot[0] <= limit and free_slot[1] >= limit \
-                                + get_seconds(walltime):
+                            if free_slot[0] <= limit and free_slot[1] >= limit + walltime:
                                 kavlan_free = True
                         if kavlan_free:
                             free_vlans += 1
@@ -194,8 +189,7 @@ def compute_slots(planning, walltime, excluded_elements = None):
                     if int(vlan.split('-')[1]) > 10:
                         kavlan_global_free = False
                         for free_slot in vlan_planning['free']:
-                            if free_slot[0] <= limit and free_slot[1] >= limit \
-                                + get_seconds(walltime):
+                            if free_slot[0] <= limit and free_slot[1] >= limit  + walltime:
                                 kavlan_global_free = True
                         if kavlan_global_free:
                             free_vlans_global.append(site)
@@ -204,7 +198,7 @@ def compute_slots(planning, walltime, excluded_elements = None):
 
                 ## MISSING OTHER RESOURCES COMPUTATION
 
-        slots.append( [ limit, limit +get_seconds(walltime), free_elements] )
+        slots.append( [ limit, limit + walltime, free_elements] )
 
     slots.sort()
     return slots
@@ -587,8 +581,8 @@ def _get_site_planning_API(site, site_planning):
                     site_planning['subnets'][subnet]['busy'].append( (start_time, end_time))
             # STORAGE IS MISSING
     except Exception, e:
-        print "except: %s" % (e,)
-        logger.warn('Error with '+site+', removing from computation')
+        logger.warn('error connecting to oar database / getting planning from ' + site)
+        logger.detail("exception:\n" + format_exc())
         currentThread().broken = True
 
 def _get_planning_API(planning):
@@ -609,101 +603,105 @@ def _get_planning_API(planning):
         del planning[site]
 
 def _get_site_planning_MySQL(site, site_planning):
-    try:
-        db = MySQLdb.connect( host = 'mysql.'+site+'.grid5000.fr', port = 3306, user = 'oarreader',
-                              passwd = 'read', db = 'oar2', connect_timeout = 3)
+    with G5kAutoPortForwarder(site,
+                              'mysql.' + site + '.grid5000.fr',
+                              g5k_configuration['oar_mysql_ro_port']) as (host, port):
         try:
+            db = MySQLdb.connect(host = host, port = port,
+                                 user = g5k_configuration['oar_mysql_ro_user'],
+                                 passwd = g5k_configuration['oar_mysql_ro_password'],
+                                 db = g5k_configuration['oar_mysql_ro_db'])
+            try:
 
-            # Change the group_concat_max_len to retrive long hosts lists
-            db.query('SET SESSION group_concat_max_len=102400')
+                # Change the group_concat_max_len to retrive long hosts lists
+                db.query('SET SESSION group_concat_max_len=102400')
 
 # CHUNKS IS NOT FINISHED BUT WE KEEP THE REQUEST FOR LATER
-#            db.query("""SELECT *
-#                FROM information_schema.COLUMNS
-#                WHERE TABLE_SCHEMA = 'oar2'
-#                AND TABLE_NAME = 'resources'
-#                AND COLUMN_NAME = 'chunks'
-#                LIMIT 1""")
-#            r = db.store_result()
-#            if len( r.fetch_row( maxrows = 0, how=1) )> 0:
-#                has_chunks = True
-#            else:
-#            has_chunks = False
-#            if has_chunks:
-#                sql += ", IF(R.type = 'storage', R.chunks, null) as storage "
+#                db.query("""SELECT *
+#                    FROM information_schema.COLUMNS
+#                    WHERE TABLE_SCHEMA = 'oar2'
+#                    AND TABLE_NAME = 'resources'
+#                    AND COLUMN_NAME = 'chunks'
+#                    LIMIT 1""")
+#                r = db.store_result()
+#                if len( r.fetch_row( maxrows = 0, how=1) )> 0:
+#                    has_chunks = True
+#                else:
+#                has_chunks = False
+#                if has_chunks:
+#                    sql += ", IF(R.type = 'storage', R.chunks, null) as storage "
 
-            # Retrieving alive resources
-            sql = """SELECT DISTINCT IF(R.type = 'default',R.network_address,null) as host,
-                IF(R.type = 'kavlan' or R.type = 'kavlan-global', R.vlan,null) as vlans,
-                IF(R.type = 'subnet', R.subnet_address, null) as subnet
-                FROM resources R
-                WHERE state <> 'Dead'; """
+                # Retrieving alive resources
+                sql = """SELECT DISTINCT IF(R.type = 'default',R.network_address,null) as host,
+                    IF(R.type = 'kavlan' or R.type = 'kavlan-global', R.vlan,null) as vlans,
+                    IF(R.type = 'subnet', R.subnet_address, null) as subnet
+                    FROM resources R
+                    WHERE state <> 'Dead';"""
+                db.query(sql)
+                r = db.store_result()
+                for data in r.fetch_row(maxrows = 0, how=1):
+                    if data['host'] is not None:
+                        cluster = data['host'].split('-')[0]
+                        if cluster in site_planning:
+                            site_planning[cluster][data['host']] = {'busy': [], 'free': []}
+                    if 'vlans' in site_planning and data['vlans'] is not None:
+                        site_planning['vlans']['kavlan-'+data['vlans']] = {'busy': [], 'free': []}
+                    if site_planning.has_key('subnets') and data['subnet'] is not None:
+                        site_planning['subnets'][data['subnet']] = {'busy': [], 'free': []}
+                    # STORAGE WILL BE ADDED LATER
 
-            db.query(sql)
-            r = db.store_result()
-            for data in r.fetch_row( maxrows = 0, how=1 ):
-                if data['host'] is not None:
-                    cluster = data['host'].split('-')[0]
-                    site_planning[cluster][data['host']] = {'busy': [], 'free': []}
-                if site_planning.has_key('vlans') and data['vlans'] is not None:
-                    site_planning['vlans']['kavlan-'+data['vlans']] = {'busy': [], 'free': []}
-                if site_planning.has_key('subnets') and data['subnet'] is not None:
-                    site_planning['subnets'][data['subnet']] = {'busy': [], 'free': []}
-                # STORAGE WILL BE ADDED LATER
+                sql = """SELECT J.job_id, J.state, GJP.start_time AS start_time, J.job_user AS user,
+                GJP.start_time+MJD.moldable_walltime+TIME_TO_SEC('0:01:05') AS stop_time,
+                GROUP_CONCAT(DISTINCT R.network_address) AS hosts,
+                GROUP_CONCAT(DISTINCT  R.vlan ) AS vlan,
+                GROUP_CONCAT(DISTINCT R.subnet_address) AS subnets
+                FROM jobs J
+                LEFT JOIN moldable_job_descriptions MJD
+                    ON MJD.moldable_job_id=J.job_id
+                LEFT JOIN gantt_jobs_predictions GJP
+                    ON GJP.moldable_job_id=MJD.moldable_id
+                INNER JOIN gantt_jobs_resources AR
+                    ON AR.moldable_job_id=MJD.moldable_id
+                LEFT JOIN resources R
+                    ON AR.resource_id=R.resource_id
+                WHERE ( J.state='Launching' OR J.state='Running' OR J.state='Waiting')
+                    AND queue_name<>'besteffort'
+                GROUP BY J.job_id
+                ORDER BY J.start_time, R.network_address,
+                    CONVERT(SUBSTRING_INDEX(SUBSTRING_INDEX(R.network_address,'.',1),'-',-1), SIGNED)"""
+                db.query(sql)
+                r = db.store_result()
+                for job in r.fetch_row( maxrows = 0, how=1 ):
+                    if job['hosts']:
+                        for host in job['hosts'].split(','):
+                            if host != '':
+                                cluster = host.split('-')[0]
+                                if cluster in site_planning:
+                                    if site_planning[cluster].has_key(host):
+                                        site_planning[cluster][host]['busy'].append( (int(job['start_time']), \
+                                                                                          int(job['stop_time'])))
+                    if site_planning.has_key('vlans') and job['vlan'] is not None:
+                        ##HACK TO FIX BUGS IN LILLE, SOPHIA, RENNES OAR2 DATABASE
+                        try:
+                            vlan = int(job['vlan'])
+                            # We are only interested in routed vlan
+                            if vlan > 3:
+                                site_planning['vlans']['kavlan-'+job['vlan']]['busy'].append( (int(job['start_time']),\
+                                                                                            int(job['stop_time'])) )
+                        except:
+                            pass
 
-            sql = """SELECT J.job_id, J.state, GJP.start_time AS start_time, J.job_user AS user,
-            GJP.start_time+MJD.moldable_walltime+TIME_TO_SEC('0:01:05') AS stop_time,
-            GROUP_CONCAT(DISTINCT R.network_address) AS hosts,
-            GROUP_CONCAT(DISTINCT  R.vlan ) AS vlan,
-            GROUP_CONCAT(DISTINCT R.subnet_address) AS subnets
-            FROM jobs J
-            LEFT JOIN moldable_job_descriptions MJD
-                ON MJD.moldable_job_id=J.job_id
-            LEFT JOIN gantt_jobs_predictions GJP
-                ON GJP.moldable_job_id=MJD.moldable_id
-            INNER JOIN gantt_jobs_resources AR
-                ON AR.moldable_job_id=MJD.moldable_id
-            LEFT JOIN resources R
-                ON AR.resource_id=R.resource_id
-            WHERE ( J.state='Launching' OR J.state='Running' OR J.state='Waiting')
-                AND queue_name<>'besteffort'
-            GROUP BY J.job_id
-            ORDER BY J.start_time, R.network_address,
-                CONVERT(SUBSTRING_INDEX(SUBSTRING_INDEX(R.network_address,'.',1),'-',-1), SIGNED)"""
-
-            db.query(sql)
-            r = db.store_result()
-            for job in r.fetch_row( maxrows = 0, how=1 ):
-                if job['hosts'] != '':
-                    for host in job['hosts'].split(','):
-                        if host != '':
-                            cluster = host.split('-')[0]
-                            if site_planning[cluster].has_key(host):
-                                site_planning[cluster][host]['busy'].append( (int(job['start_time']), \
-                                                                               int(job['stop_time'])))
-                if site_planning.has_key('vlans') and job['vlan'] is not None:
-                    ##HACK TO FIX BUGS IN LILLE, SOPHIA, RENNES OAR2 DATABASE
-                    try:
-                        vlan =  int(job['vlan'])
-                        # We are only interested in routed vlan
-                        if vlan > 3:
-                            site_planning['vlans']['kavlan-'+job['vlan']]['busy'].append( (int(job['start_time']),\
-                                                                                        int(job['stop_time'])) )
-                    except:
-                        pass
-
-
-                if site_planning.has_key('subnets') and job['subnets'] is not None:
-
-                    for subnet in job['subnets'].split(','):
-                        site_planning['subnets'][subnet]['busy'].append( (int(job['start_time']), \
-                                                                               int(job['stop_time'])))
-                # MISSING STORAGE
-        finally:
-            db.close()
-    except:
-        logger.warn('error with '+site+', removing from computation')
-        currentThread().broken = True
+                    if site_planning.has_key('subnets') and job['subnets'] is not None:
+                        for subnet in job['subnets'].split(','):
+                            site_planning['subnets'][subnet]['busy'].append( (int(job['start_time']), \
+                                                                                   int(job['stop_time'])))
+                    # MISSING STORAGE
+            finally:
+                db.close()
+        except Exception, e:
+            logger.warn('error connecting to oar database / getting planning from ' + site)
+            logger.detail("exception:\n" + format_exc())
+            currentThread().broken = True
 
 def _get_planning_MySQL(planning):
     """Retrieve the planning using the oar2 database"""
@@ -792,7 +790,6 @@ def _trunc_el_planning(el_planning, starttime, endtime):
         el_planning.sort()
 
 
-
 def _fill_el_planning_free(el_planning, starttime, endtime):
     """An internal function to compute the planning free of all elements"""
     if len(el_planning['busy']) > 0:
@@ -805,96 +802,34 @@ def _fill_el_planning_free(el_planning, starttime, endtime):
     else:
             el_planning['free'].append((starttime, endtime))
 
+
 def _slots_limits(planning):
     """Return the limits of slots, defined by a resource state change."""
-    limits = []
+    limits = set()
     for site in planning.itervalues():
         for res_pl in site.itervalues():
             for el_planning in res_pl.itervalues():
                     for start, stop in el_planning['busy']:
-                        if start not in limits:
-                            limits.append(start)
-                        if stop not in limits:
-                            limits.append(stop)
-
+                        limits.add(start)
+                        limits.add(stop)
                     for start, stop in el_planning['free']:
-                        if start not in limits:
-                            limits.append(start)
-                        if stop not in limits:
-                            limits.append(stop)
+                        limits.add(start)
+                        limits.add(stop)
     limits = sorted(limits)
-    limits.pop()
+    if len(limits) > 0:
+        limits.pop()
     return limits
 
 
-
 def _add_charter_to_planning(planning, starttime, endtime):
     charter_el_planning = get_charter_el_planning(starttime, endtime)
 
     for site in planning.itervalues():
         for res_pl in site.itervalues():
             for el_planning in res_pl.values():
-                el_planning['busy'].sort()
                 el_planning['busy'] += charter_el_planning
                 el_planning['busy'].sort()
 
-
-
-def g5k_charter_time(t):
-    """Is the given date in a g5k charter time period ?
-
-    Returns a boolean, True if the given date is in a period where
-    the g5k charter needs to be respected, False if it is in a period
-    where charter is not applicable (night, weekends)
-
-    :param t: a date in a type supported by `execo.time_utils.get_unixts`
-    """
-    dt = unixts_to_datetime(get_unixts(t))
-    if dt.weekday() in [5, 6]: return False
-    if dt.hour < 9 or dt.hour >= 19: return False
-    return True
-
-def get_next_charter_period(start, end):
-    """Return the next g5k charter time period.
-
-    :param start: date in a type supported by
-      `execo.time_utils.get_unixts` from which to start searching for
-      the next g5k charter time period. If start is in a g5k charter
-      time period, the returned g5k charter time period starts at
-      start.
-
-    :param end: date in a type supported by
-      `execo.time_utils.get_unixts` until which to search for the next
-      g5k charter time period. If end is in the g5k charter time
-      period, the returned g5k charter time period ends at end.
-
-    :returns: a tuple (charter_start, charter_end) of
-      unix timestamps. (None, None) if no g5k charter time period
-      found
-    """
-    start = unixts_to_datetime(get_unixts(start))
-    end = unixts_to_datetime(get_unixts(end))
-    if end <= start:
-        return None, None
-    elif g5k_charter_time(start):
-        charter_end = start.replace(hour = 19, minute = 0, second = 0, microsecond = 0)
-        if charter_end > end: charter_end = end
-        return datetime_to_unixts(start), datetime_to_unixts(charter_end)
-    else:
-        wd = start.weekday()
-        if (wd == 4 and start.hour >= 19) or (wd > 4):
-            charter_start = start.replace(hour = 9, minute = 0, second = 0, microsecond = 0) + timedelta(days = 7 - wd)
-        elif start.hour < 9:
-            charter_start = start.replace(hour = 9, minute = 0, second = 0, microsecond = 0)
-        else:
-            charter_start = start.replace(hour = 9, minute = 0, second = 0, microsecond = 0) + timedelta(days = 1)
-        if charter_start > end:
-            return None, None
-        charter_end = charter_start.replace(hour = 19, minute = 0, second = 0, microsecond = 0)
-        if charter_end > end:
-            charter_end = end
-        return datetime_to_unixts(charter_start), datetime_to_unixts(charter_end)
-
 def get_charter_el_planning(start_time, end_time):
     """Returns the list of tuples (start, end) of g5k charter time periods between start_time and end_time.
 
